// @ myth - common - common.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/common.h"

// @@ GNU fix

#ifdef __GNUG__
pfVV set_new_handler(pfVV handler)
{ }
#endif

// @@ direction

// @@@ const int dx[], dy[], dz[]

const int dx[DIRECTION_END] = 
{ 0,  0,  1,  1,  1,  0, -1, -1, -1,  0,  0 };

const int dy[DIRECTION_END] =
{ 0, -1, -1,  0,  1,  1,  1,  0, -1,  0,  0 };

const int dz[DIRECTION_END] =
{ 0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  1 };

// @@ coor

// @@@ region

// @@@@ bool region::check()

bool region::check()
{
    if(x == to.x)
    {
	x = from.x;
	
	if(to.y > y + 1)
	    y++;
	else if(to.y < y - 1)
	    y--;
	else
	    return FALSE;
    }
    else if(y == to.y)
    {
	y = from.y;
	
	if(to.x > x + 1)
	    x++;
	else if(to.x < x - 1)
	    x--;
	else
	    return FALSE;
    }
    return TRUE;
}

// @@@@ void region::NextX()

void region::NextX()
{
    if(to.x < x)
	x--;
    else
	x++;
}

// @@@@ void region::NextY()

void region::NextY()
{
    if(to.y < y)
	y--;
    else
	y++;
}

// @@@ const coor& region::operator=(const region& reg)

const coor& region::operator=(const region& reg)
{
    x = reg.x;
    y = reg.y;
    from = reg.from;
    to = reg.to;
    
    return reg;
}

// @@@@ region::region()

region::region()
{ from = to = NoWhere; }

// @@@@ region::regin(const coor& reg)

region::region(const region& reg)
{
    from.x = x = reg.from.x;
    from.y = y = reg.from.y;
    to.x = reg.to.x;
    to.y = reg.to.y;
}

// @@@@ region::region(const coor& start, const coor& target)

region::region(const coor& start, const coor& target) : (start)
{
    from.x = x = start.x;
    from.y = y = start.y;
    to.x = target.x;
    to.y = target.y;
}

// @@@ straight
//
// Stolen from The C++ Programming Language

// @@@@ void straight::next()

void straight::next()
{
    if(coor(x, y) == to)
	done = TRUE;
    else
    {
	if(epsilon <= xcrit)
	{
	    x += step.x;
	    epsilon += 2 * diff.y;
	}
	if(epsilon >= diff.x || diff.x <= diff.y)
	{
	    y += step.y;
	    epsilon -= 2 * diff.x;
	}
    }
}

// @@@@ straight::straight(const coor& start, const coor& target)

straight::straight(const coor& start, const coor& target) : (start)
{
    to = target;
    diff = abs(target - start);
    step = coor(target.x < start.x ? -1 : 1, target.y < start.y ? -1 : 1);
    xcrit = 2 * diff.x - diff.y;
    epsilon = 0;
    done = FALSE;
}

// @@ function

// @@@ int log(long l)
//
// Make large numbers small...

int log(long l)
{
    int power = 0;
    
    while(l > 1)
    {
	l /= 2;
	power++;
    }
    return power;
}

// @@@ int MyLog(long l)

int MyLog(long l)
{
    if(l < 2)
	return 1;

    for(int log = 1; l > 1; l /= 2)
	;

    return log;
}

// @@@ bool NextTo(coor a, coor b)
//
// Return true if a and b is next to each other

bool NextTo(coor a, coor b)
{
    coor c = abs(a - b);

    return max(c.x, c.y) == 1;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
