// @ myth - common - picture.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// The PICTURE class handle two dimensional arrays of charecters.
//
// To create a PICTURE, declare it.
//    PICTURE paint(coor(20, 10));
//
// It is initialised with spaces. You may read and modify the array
// with the usual commands:
//    paint[coor(15, 5)] = paint[coor(0, 7)];
//
// The current size of the array can be read by:
//    coor HowBig = paint.size();
//
// It can be resized (and cleared) by:
//    paint.resize(coor(40, 10));

// @@ include

#include "common/picture.h"
#include "common/error.h"

// @@ class

// @@@ void PICTURE::free()
//
// Free any storage allocated to the array.

void PICTURE::free()
{
    if(buffer)
    {
	for(int j = 0; j < Size.y; j++)
	    delete[Size.x] buffer[j];
	delete[Size.y] buffer;
	buffer = NULL;
	Size = Origo;
    }
}

// @@@ * void PICTURE::resize(coor size, char fill = ' ')
//
// Substitute the array, with a new empty one.

void PICTURE::resize(coor size, char fill = ' ')
{
    free();
    buffer = new char*[size.y];
    for(int j = 0; j < size.y; j++)
    {
	buffer[j] = new char[size.x];
	for(int i = 0; i < size.x; i++)
	    buffer[j][i] = fill;
    }
    Size = size;
}

// @@@ const PICTURE& PICTURE::operator=(PICTURE& picture)

const PICTURE& PICTURE::operator=(const PICTURE& picture)
{
    if(this != &picture)
    {
	free();
	buffer = new char*[picture.size().y];
	coor p;
	for(p.y = 0; p.y < picture.size().y; p.y++)
	{
	    buffer[p.y] = new char[picture.size().x];
	    for(p.x = 0; p.x < picture.size().x; p.x++)
		buffer[p.y][p.x] = picture[p];
	}
	Size = picture.size();
    }
    return picture;
}

// @@@ PICTURE::PICTURE(const PICTURE& picture)

PICTURE::PICTURE(const PICTURE& picture)
{
    buffer = new char*[picture.size().y];
    coor p;
    for(p.y = 0; p.y < picture.size().y; p.y++)
    {
	buffer[p.y] = new char[picture.size().x];
	for(p.x = 0; p.x < picture.size().x; p.x++)
	    buffer[p.y][p.x] = picture[p];
    }
    Size = picture.size();
}

// @@@ PICTURE::PICTURE()

PICTURE::PICTURE()
{
    buffer = NULL;
    Size = Origo;
}

// @@@ PICTURE::PICTURE(coor size, char fill = ' ')

PICTURE::PICTURE(coor size, char fill = ' ')
{
    buffer = NULL;
    resize(size, fill);
}

// @@@ PICTURE::~PICTURE()

PICTURE::~PICTURE()
{
    free();
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
